# -*- coding: utf-8 -*-
# (c) 2014-2019 The mqttwarn developers
from __future__ import print_function
import os
import sys
import json
import codecs
import signal
import logging

from docopt import docopt

from mqttwarn import __version__
from mqttwarn.configuration import load_configuration
from mqttwarn.core import bootstrap, connect, cleanup, run_plugin
from mqttwarn.util import get_resource_content

logger = logging.getLogger(__name__)

APP_NAME = 'mqttwarn'


def run():
    """
    Usage:
      {program} [make-config]
      {program} [make-samplefuncs]
      {program} [--plugin=] [--data=]
      {program} --version
      {program} (-h | --help)

    Configuration file options:
      make-config               Will dump configuration file content to STDOUT,
                                suitable for redirecting into a configuration file.

    Miscellaneous options:
      --version                 Show version information
      -h --help                 Show this screen

    """

    # Use generic commandline options schema and amend with current program name
    commandline_schema = run.__doc__.format(program=APP_NAME)

    # Read commandline options
    options = docopt(commandline_schema, version=APP_NAME + ' ' + __version__)

    # Python2/3 string encoding compat - sigh.
    # https://stackoverflow.com/questions/2737966/how-to-change-the-stdin-and-stdout-encoding-on-python-2/58449987#58449987
    utf8_writer = codecs.getwriter('utf-8')
    if sys.version_info.major <= 2:
        sys.stdout = utf8_writer(sys.stdout)
    else:
        sys.stdout = utf8_writer(sys.stdout.buffer)

    if options['make-config']:
        payload = get_resource_content('mqttwarn.examples', 'basic/mqttwarn.ini')
        print(payload)

    elif options['make-samplefuncs']:
        payload = get_resource_content('mqttwarn.examples', 'basic/samplefuncs.py')
        print(payload)

    elif options['--plugin'] and options['--data']:

        # Decode arguments
        plugin = options['--plugin']
        data = json.loads(options['--data'])

        # Launch service plugin in standalone mode
        launch_plugin_standalone(plugin, data)


    # Run mqttwarn in service mode when no command line arguments are given
    else:
        run_mqttwarn()


def launch_plugin_standalone(plugin, data):
    # Load configuration file
    scriptname = os.path.splitext(os.path.basename(sys.argv[0]))[0]
    config = load_configuration(name=scriptname)

    # Setup logging
    setup_logging(config)
    logger.info('Running service plugin "{}" with data "{}"'.format(plugin, data))

    # Launch service plugin
    run_plugin(config=config, name=plugin, data=data)


def run_mqttwarn():

    # Script name (without extension) used as last resort fallback for config/logfile names
    scriptname = os.path.splitext(os.path.basename(sys.argv[0]))[0]

    # Load configuration file
    config = load_configuration(name=scriptname)

    # Setup logging
    setup_logging(config)
    logger.info("Starting {}".format(scriptname))
    logger.info("Log level is %s" % logging.getLevelName(logger.getEffectiveLevel()))

    # Handle signals
    signal.signal(signal.SIGTERM, cleanup)
    signal.signal(signal.SIGINT, cleanup)

    # Bootstrap mqttwarn.core
    bootstrap(config=config, scriptname=scriptname)

    # Connect to broker and start listening
    connect()


def setup_logging(config):
    LOGLEVEL = config.loglevelnumber
    LOGFILE = config.logfile
    LOGFORMAT = config.logformat

    # Send log messages to sys.stderr by configuring "logfile = stream://sys.stderr"
    if LOGFILE.startswith('stream://'):
        LOGFILE = LOGFILE.replace('stream://', '')
        logging.basicConfig(stream=eval(LOGFILE), level=LOGLEVEL, format=LOGFORMAT)

    # Send log messages to file by configuring "logfile = 'mqttwarn.log'"
    else:
        logging.basicConfig(filename=LOGFILE, level=LOGLEVEL, format=LOGFORMAT)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Halacs <halacs87()gmail.com>'
__copyright__ = 'Copyright 2018 Halacs'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import MySQLdb


# https://mail.python.org/pipermail/tutor/2010-December/080701.html
def add_row(srv, cursor, tablename, rowdict):
    # XXX tablename not sanitized
    # XXX test for allowed keys is case-sensitive

    unknown_keys = None

    # filter out keys that are not column names
    cursor.execute("describe %s" % tablename)
    allowed_keys = set(row[0] for row in cursor.fetchall())
    keys = allowed_keys.intersection(rowdict)

    if len(rowdict) > len(keys):
        unknown_keys = set(rowdict) - allowed_keys

    columns = ", ".join(keys)
    values_template = ", ".join(["%s"] * len(keys))

    sql = "insert into %s (%s) values (%s)" % (tablename, columns, values_template)
    values = tuple(rowdict[key] for key in keys)
    cursor.execute(sql, values)

    return unknown_keys


def daraFv(srv, item, data, col_data, mapping):
    if data is not None:
        for key in list(data.keys()):
            if type(data[key]) is dict:
                daraFv(srv, item, data[key], col_data, mapping)
            else:
                if key in mapping:
                    try:
                        col_data[mapping[key]] = data[key].format(**data).encode('utf-8')
                    except Exception as e:
                        col_data[mapping[key]] = data[key]


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    host    = item.config.get('host', 'localhost')
    port    = item.config.get('port', 3306)
    user    = item.config.get('user')
    passwd  = item.config.get('pass')
    dbname  = item.config.get('dbname')

    try:
        table_name = item.addrs[0].format(**item.data).encode('utf-8')
        mapping = item.addrs[1]
        static = item.addrs[2]
    except:
        srv.logging.warn("halsql target incorrectly configured.")
        return False

    try:
        conn = MySQLdb.connect(host=host, user=user, passwd=passwd, db=dbname)
        cursor = conn.cursor()
    except Exception as e:
        srv.logging.warn("Cannot connect to mysql: %s" % e)
        return False

    col_data = {}

    # dynamic data part: remapp keys comes from the message
    daraFv(srv, item, item.data, col_data, mapping)
    
    # static data: add static key/value pairs to col_data
    if static is not None:
        col_data = dict(list(col_data.items()) + list(static.items()))

    try:
        unknown_keys = add_row(srv, cursor, table_name, col_data)
        if unknown_keys is not None:
            srv.logging.debug("Skipping unused keys %s" % ",".join(unknown_keys))
        conn.commit()
    except Exception as e:
        srv.logging.warn("Cannot add mysql row: %s" % e)
        cursor.close()
        conn.close()
        return False

    cursor.close()
    conn.close()

    return True

# vim: tabstop=4 expandtab



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__    = 'Bram Hendrickx'
__copyright__ = 'Copyright 2016 Bram Hendrickx'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

Original script by Bram Hendrickx. https://github.com/jpmens/mqttwarn/blob/master/services/ifttt.py
Modified to work with the autoremote api https://joaoapps.com/autoremote/
"""

__author__    = 'Michael Brougham'
__copyright__ = 'Copyright 2018 Michael Brougham'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import requests


def plugin(srv, item):
    ''' expects (apikey, password, target, group, ttl) in addrs '''

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    try:
        srv.logging.debug("Sending to autoremote service")
        params = {
            'key': item.addrs[0],
            'message': item.message,
            'target': item.addrs[2],
            'sender': item.topic,
            'password': item.addrs[1],
            'ttl': item.addrs[4],
            'collapseKey': item.addrs[3],
        }
        requests.get('https://autoremotejoaomgcd.appspot.com/sendmessage', params=params)
        srv.logging.debug("Successfully sent to autoremote service")

    except Exception as e:
        srv.logging.warning("Failed to send message to autoremote service" % e)
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import socket


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    try:
        addr, port, channel = item.addrs
    except:
        srv.logging.warn("Incorrect target configuration")
        return False

    message = item.message

    color = None
    priority = item.priority
    if priority == 1:
        color = '%GREEN'
    if priority == 2:
        color = '%RED'

    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((addr, port))
        if color is not None:
            sock.send(color)
        sock.send(message)
        sock.close()

    except Exception as e:
        srv.logging.error("Error sending IRCCAT notification to %s:%s [%s]: %s" % (item.target, addr, port, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from six import string_types

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'


def plugin(srv, item):
    """ Publish via MQTT to the same broker connection.
        Requires topic, qos and retain flag """

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    outgoing_topic =  item.addrs[0]
    qos  =  item.addrs[1]
    retain = item.addrs[2]

    # Attempt to interpolate data into topic name. If it isn't possible
    # ignore, and return without publish

    if item.data is not None:
        try:
            outgoing_topic =  item.addrs[0].format(**item.data).encode('utf-8')
        except:
            srv.logging.debug("Outgoing topic cannot be formatted; not published")
            return False

    outgoing_payload = item.message
    if isinstance(outgoing_payload, string_types):
        outgoing_payload = bytearray(outgoing_payload, encoding='utf-8')

    try:
        srv.mqttc.publish(outgoing_topic, outgoing_payload, qos=qos, retain=retain)
    except Exception as e:
        srv.logging.warning("Cannot PUBlish via `mqttpub:%s': %s" % (item.target, e))
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2018 The mqttwarn developers
from builtins import str
import logging

from mqttwarn.core import make_service, decode_payload
from tests import configfile
from tests.util import core_bootstrap, send_message


def test_make_service():
    service = make_service(name='foo')
    assert '<mqttwarn.core.Service object at' in str(service)


def test_bootstrap(caplog):
    """
    Bootstrap the core machinery without MQTT
    """

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT
        core_bootstrap(configfile=configfile)

        # Proof that mqttwarn loaded all services properly
        assert 'Successfully loaded service "file"' in caplog.text, caplog.text
        assert 'Successfully loaded service "log"' in caplog.text, caplog.text

        assert 'Starting 1 worker threads' in caplog.text, caplog.text

        # Capturing the last message does not work. Why?
        #assert 'Job queue has 0 items to process' in caplog.text, caplog.text


def test_decode_payload_foo(caplog):

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT.
        core_bootstrap(configfile=configfile)

        # Proof that decoding an unconfigured thing yields nothing sensible.
        outcome = decode_payload(section='foo', topic='bar', payload='baz')
        assert outcome['topic'] == 'bar'
        assert outcome['payload'] == 'baz'
        assert 'Cannot decode JSON object, payload=baz' in caplog.text, caplog.text


def test_decode_payload_json(caplog):

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT.
        core_bootstrap(configfile=configfile)

        # Proof that decoding a valid JSON payload decodes it appropriately.
        outcome = decode_payload(section='foo', topic='bar', payload='{"baz": "qux"}')
        assert outcome['topic'] == 'bar'
        assert outcome['payload'] == '{"baz": "qux"}'
        assert outcome['baz'] == 'qux'


def test_decode_payload_datamap(caplog):

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT.
        core_bootstrap(configfile=configfile)

        # Proof that decoding a valid JSON payload decodes it appropriately.
        outcome = decode_payload(section='test/datamap', topic='bar', payload='{"baz": "qux"}')
        assert outcome['topic'] == 'bar'
        assert outcome['baz'] == 'qux'
        assert outcome['datamap-key'] == 'datamap-value'


def test_decode_payload_alldata(caplog):

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT.
        core_bootstrap(configfile=configfile)

        # Proof that decoding a valid JSON payload decodes it appropriately.
        outcome = decode_payload(section='test/alldata', topic='bar', payload='{"baz": "qux"}')
        assert outcome['topic'] == 'bar'
        assert outcome['baz'] == 'qux'
        assert outcome['alldata-key'] == 'alldata-value'


def test_message_basic(caplog):

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT
        core_bootstrap(configfile=configfile)

        # Signal mocked MQTT message to the core machinery for processing
        send_message(topic='test/log-1', payload='{"name": "temperature", "value": 42.42}')

        # Proof that the message has been routed to the "log" plugin properly
        assert "u'temperature: 42.42" in caplog.text, caplog.text



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Jan-Piet Mens <jp@mens.de>'
__copyright__ = 'Copyright 2018 Jan-Piet Mens'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from six import string_types

import requests

try:
    import simplejson as json
except ImportError:
    import json


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    hook_url = item.addrs[0]
    channel = item.addrs[1]
    username = item.addrs[2]  # may be None
    icon_url = item.addrs[3]  # may be None

    text = item.message
    try:
        """ Try to format a Markdown table if we have JSON in the payload """
        """ ETOOMESSY; volunteers to refactor? """
        title = item.get('title', None)
        j = json.loads(text)
        keylen = vallen = 10
        for key in j:
            # print type(key), keylen, len(key)
            if isinstance(key, string_types) and keylen < len(key):
                keylen = len(key)
            if isinstance(j[key], string_types) and vallen < len(j[key]):
                vallen = len(j[key])
        s = ""
        if title is not None and title != "":
            s = "## %s\n" % title
        key = "key"
        val = "value"
        s = s + "| {0:<{kw}}  | {1:<{vw}} |\n".format("key", "value", kw=keylen, vw=vallen)
        s = s + "|:{0:<{kw}}  |:{1:<{vw}} |\n".format('-' * keylen, '-' * vallen, kw=keylen, vw=vallen)
        for key in j:
            s = s + "| {0:<{kw}}  | {1:<{vw}} |\n".format(key, j[key], kw=keylen, vw=vallen)
        text = s
    except Exception as e:
        srv.logging.debug("not JSON; proceeding with text")
        pass

    payload = {}
    payload["channel"] = channel
    payload["text"] = text
    if username is not None:
        payload["username"] = username
    if icon_url is not None:
        payload["icon_url"] = icon_url

    # print payload

    headers = {
        "Content-type": "application/json",
        "Accept": "application/json"
    }

    try:
        r = requests.post(hook_url, data=json.dumps(payload), headers=headers)
        if r.status_code != requests.codes.ok:
            srv.logging.warning("Invalid response from Mattermost Webhook: %s" % (r.text))
            return False
    except Exception as e:
        srv.logging.warning("Failed to POST request to Mattermost Webhook: %s" % e)
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2014-2019 The mqttwarn developers
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

requires = [
    'six==1.13.0',
    'paho-mqtt==1.5.0',
    'jinja2==2.10.3',
    'attrs==19.3.0',
    'docopt==0.6.2',
    'requests==2.22.0',
    'future==0.18.2',
    'configparser==4.0.2',
]

extras = {
    'amqp': [
        'puka==0.0.7',
    ],
    'apns': [
        'apns==2.0.1',
    ],
    'asterisk': [
        'pyst2==0.5.0',
    ],
    'celery': [
        'celery',
    ],
    'dnsupdate': [
        'dnspython==1.15.0',
    ],
    'fbchat': [
        'fbchat==1.3.6',
    ],
    'gss': [
        'gdata==2.0.18',
    ],
    'gss2': [
        'gspread==2.1.1',
        'oauth2client==4.1.2',
        #'google-api-python-client==1.7.11',
    ],
    'iothub': [
        'iothub-client==1.1.2.0',
    ],
    'mysql': [
        'mysql',
    ],
    'nma': [
        'PyNMA==1.0',
    ],
    'nsca': [
        'pynsca==1.6',
    ],
    'osxnotify': [
        'pync==1.6.1',
    ],
    'pastebinpub': [
        'Pastebin==1.1.2',
    ],
    'postgres': [
        'psycopg2==2.7.4',
    ],
    'prowl': [
        'prowlpy>=0.52',
    ],
    'pushbullet': [
        'PushbulletPythonLibrary==2.3',
    ],
    'redispub': [
        'redis==2.10.6',
    ],
    'rrdtool': [
        'rrdtool==0.1.12',
    ],
    'serial': [
        'pyserial==3.4',
    ],
    'slack': [
        'slacker==0.9.65',
    ],
    'ssh': [
        'paramiko==2.4.1',
    ],
    'tootpaste': [
        'Mastodon.py==1.2.2',
    ],
    'twilio': [
        'twilio==6.11.0',
    ],
    'twitter': [
        'python-twitter==3.4.1',
    ],
    'websocket': [
        'websocket-client==0.47.0',
    ],
    'xively': [
        'xively-python',
    ],
    'xmpp': [
        'xmpppy==0.5.0rc1',
    ],
    'test': [
        'pytest==4.6.7',
        'pytest-cov==2.8.1',
        'lovely.testlayers>=0.7.1',
        'tox==3.14.2',
    ],
}

setup(name='mqttwarn',
      version='0.13.6',
      description='mqttwarn - subscribe to MQTT topics and notify pluggable services',
      long_description=README,
      license="EPL 2.0",
      classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Environment :: Plugins",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Manufacturing",
        "Intended Audience :: Science/Research",
        "Intended Audience :: System Administrators",
        "Intended Audience :: Telecommunications Industry",
        "License :: OSI Approved :: Eclipse Public License 2.0 (EPL-2.0)",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Operating System :: MacOS",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Communications",
        "Topic :: Education",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: XMPP",
        "Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator",
        "Topic :: Software Development :: Embedded Systems",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Pre-processors",
        "Topic :: Software Development :: Testing",
        "Topic :: System :: Archiving",
        "Topic :: System :: Distributed Computing",
        "Topic :: System :: Monitoring",
        "Topic :: System :: Networking :: Monitoring",
        "Topic :: System :: Systems Administration",
        "Topic :: Text Processing",
        "Topic :: Utilities",
      ],
      author='Jan-Piet Mens, Ben Jones, Andreas Motl',
      author_email='jpmens@gmail.com',
      url='https://github.com/jpmens/mqttwarn',
      keywords='mqtt notification plugins data acquisition push transformation engine ' +
               'mosquitto ',
      packages=find_packages(),
      include_package_data=True,
      package_data={
        'mqttwarn': [
          '*.ini',
        ],
      },
      zip_safe=False,
      test_suite='tests',
      install_requires=requires,
      extras_require=extras,
      tests_require=extras['test'],
      dependency_links=[
          'https://github.com/jacobb/prowlpy/archive/master.tar.gz#egg=prowlpy'
      ],
      entry_points={
          'console_scripts': [
              'mqttwarn = mqttwarn.commands:run',
          ],
      },
)
